package com.qizhen.healsphere.common.pipeline;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import us.codecraft.webmagic.ResultItems;
import us.codecraft.webmagic.Task;
import us.codecraft.webmagic.pipeline.Pipeline;
import us.codecraft.webmagic.utils.FilePersistentBase;

import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class ExcelPipeline extends FilePersistentBase implements Pipeline {

    private String filename;//文件名
    private int rows = 0;//当前要编辑的行
    private HSSFWorkbook workbook;//工作蒲
    private HSSFSheet sheet;//工作表
    private List<String> titles;
    //构造方法
    public ExcelPipeline(String path,String name,List<String> titles) {
        this.titles = titles;
        //设置保存路径
        setPath(path);

        //设置文件名是日期格式
        filename = name+new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".xls";
        //创建工作薄对象
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        //创建工作表对象
        sheet = workbook.createSheet(name);

        //创建工作表的行
        HSSFRow row = sheet.createRow(rows);

        //创建标题
        for(int i=0;i<titles.size();i++) {
            row.createCell(i).setCellValue(titles.get(i));
        }
        //每写完一行我们就要开始写下一行
        rows++;


    }

    @Override
    public void process(ResultItems resultItems, Task task) {
        //创建工作表的行
        HSSFRow row = sheet.createRow(rows);
        //提取保存的内容
        for(int i=0;i<titles.size();i++) {
            //输出
            row.createCell(i).setCellValue(resultItems.get(titles.get(i))==null?"":resultItems.get(titles.get(i)).toString());
        }
        rows++;
        //写完之后保存
        save();
    }

    /**
     * 保存表格
     **/
    private synchronized void save() {
        try {
            //文档输出
            FileOutputStream out = new FileOutputStream(getFile(this.path + filename));
            workbook.write(out);
            out.close();
            System.out.println(this.path + filename + "存储完毕");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


}
