package com.qizhen.healsphere.config;

import org.neo4j.driver.v1.AuthTokens;
import org.neo4j.driver.v1.Driver;
import org.neo4j.driver.v1.GraphDatabase;
import org.neo4j.driver.v1.Session;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

@Component
public class Neo4jUtil {

    @Value("${spring.neo4j.uri}")
    private String url;

    @Value("${spring.neo4j.authentication.username}")
    private String username;

    @Value("${spring.neo4j.authentication.password}")
    private String password;

    private Driver driver;

    @Bean
    public Driver neo4jDriver() {
        driver = GraphDatabase.driver(url, AuthTokens.basic(username, password));
        return driver;
    }

    public Session getSession() {
        return driver.session();
    }

    @Component
    public static class Neo4jDriverShutdown {
        private final Driver driver;

        public Neo4jDriverShutdown(Driver driver) {
            this.driver = driver;
        }

        // 在容器关闭时关闭 Driver
        public void shutdown() {
            if (driver!= null) {
                driver.close();
            }
        }
    }
}