package com.qizhen.healsphere.repository.neo4j;

import com.qizhen.healsphere.config.Neo4jUtil;
import com.qizhen.healsphere.repository.neo4j.param.UpdatePropertyParam;
import org.neo4j.driver.v1.Session;
import org.neo4j.driver.v1.StatementResult;
import org.neo4j.driver.v1.Values;
import org.neo4j.driver.v1.types.Node;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.Map;

@Repository
public class BasePropertyRepository {

    @Autowired
    private Neo4jUtil neo4jUtil;

    /**
     * 新增实体属性
     * @param label
     * @param nodeId
     * @param properties
     * @return 实体map
     */
    public Map<String,Object> createProperty(String label, Long nodeId, Map<String, Object> properties) {
        StringBuilder setClause = new StringBuilder();
        properties.forEach((key, value) -> setClause.append("n.").append(key).append(" = '").append(value).append("'").append(", "));

        // 移除最后的逗号
        if (setClause.length() > 0) {
            setClause.setLength(setClause.length() - 2);
        }

        String query = String.format("MATCH (n:`%s`) WHERE id(n) = $nodeId SET %s RETURN n", label, setClause);

        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query, Values.parameters("nodeId", nodeId));
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;

        }
    }

    /**
     * 删除实体属性
     * @param label
     * @param nodeId
     * @param propertyName
     * @return 实体map
     */
    public Map<String,Object> deleteProperty(String label, Long nodeId,String propertyName) {
        String query = String.format("MATCH (n:`%s`) WHERE id(n) = $nodeId REMOVE n.%s RETURN n", label,propertyName);

        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("nodeId", nodeId));
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;
        }
    }

    /**
     * 更新实体属性
     *
     * @param updatePropertyParam
     * @return 实体map
     */
    public Map<String,Object> updateProperty(UpdatePropertyParam updatePropertyParam) {
        String query = String.format("MATCH (n:`%s`) WHERE id(n) = $nodeId REMOVE n.%s " +
                "SET n.%s = $newPropertyValue RETURN n", updatePropertyParam.getLabel(), updatePropertyParam.getOldPropertyName(), updatePropertyParam.getNewPropertyName());

        try(Session session = neo4jUtil.getSession()) {
            StatementResult result = session.run(query,Values.parameters("nodeId", updatePropertyParam.getNodeId(),"newPropertyValue", updatePropertyParam.getNewPropertyValue()));
            Map<String,Object> map = new HashMap<>();
            if(result.hasNext()) {
                Node node = result.single().get("n").asNode();
                map.put("id", node.id());
                map.put("labels", node.labels());
                map.put("properties", node.asMap());
            }
            return map;
        }
    }
}