package com.qizhen.healsphere.util;

import java.nio.file.Path;
import java.nio.file.Paths;

public class FileCommonUtils {

    // 定义允许的字符集，这里只允许字母、数字、下划线、连字符和汉字
    private static final String ALLOWED_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_-\\u4e00-\\u9fa5、";

    /**
     * 清理文件名中的特殊字符
     *
     * @param originalFileName 原始文件名
     * @return 清理后的文件名
     */
    public static String sanitizeFileName(String originalFileName) {
        if (originalFileName == null) {
            return null;
        }

        StringBuilder sanitizedFileName = new StringBuilder();
        for (char c : originalFileName.toCharArray()) {
            if (isAllowedChar(c)) {
                sanitizedFileName.append(c);
            }
        }

        return sanitizedFileName.toString();
    }

    /**
     * 检查字符是否在允许的字符集中
     *
     * @param c 字符
     * @return 如果字符在允许的字符集中，返回true；否则返回false
     */
    private static boolean isAllowedChar(char c) {
        return ALLOWED_CHARS.indexOf(c) != -1 || (c >= '\u4e00' && c <= '\u9fa5');
    }


    /**
     * 从文件路径中提取文件名（不包括目录和扩展名）
     *
     * @param filePath 文件路径
     * @return 文件名（不包括目录和扩展名）
     */
    public static String getFileNameWithoutExtension(String filePath) {
        Path path = Paths.get(filePath);
        String fileName = path.getFileName().toString();
        int dotIndex = fileName.lastIndexOf('.');
        if (dotIndex == -1) {
            return fileName; // 没有扩展名
        }
        return fileName.substring(0, dotIndex);
    }

    public static void main(String[] args) {
        String filePath = "E:\\project\\demo\\example.txt";
        String fileNameWithoutExtension = getFileNameWithoutExtension(filePath);
        System.out.println("File Name Without Extension: " + fileNameWithoutExtension);
    }
}