package com.qizhen.healsphere;

import com.alibaba.fastjson.JSONArray;
import com.qizhen.healsphere.common.ai.BaidubceUtil;
import com.qizhen.healsphere.common.ai.Knowlege;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.service.EntityService;
import com.qizhen.healsphere.service.RelationshipService;
import com.qizhen.healsphere.web.vo.CreateEntityVO;
import com.qizhen.healsphere.web.vo.RelationshipVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.CollectionUtils;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 英文名
 */
@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere.model", "com.qizhen.healsphere.repository"})
@SpringBootTest
public class JianchaYwmTest {
    @Autowired
    RelationshipService relationshipService;
    @Autowired
    EntityService entityService;
    private static int maxCount= 100;
    private static String directoryPath  = "C:\\Users\\17664\\Desktop\\邵逸夫医院爬取数据\\检查";
    static HSSFWorkbook workbook;
    static String startLabel = "辅助检查";
    @Test
    public void writeNeo4j() throws Exception {
        String propertys = "别名" +
                ",英文名称" +
                ",参考区间";
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        String accessToken = BaidubceUtil.getAccessToken();
        String[] split = propertys.split(",");
        for(String property:split) {
            HSSFSheet sheet = workbook.createSheet(property);
            List<Knowlege> knowleges = saveExecl(property, accessToken, sheet);

            for (Knowlege temp:knowleges) {
                String value = temp.getValue();
                if(StringUtils.isBlank(value)){
                    continue;
                }
                try{
                    JSONArray jsonArray = JSONArray.parseArray(value);
                    BaseEntity startEntity = createNoExists(startLabel, temp.getEntity());
                    long startId = startEntity.getId();
                    List<RelationshipVO> relationshipList = new ArrayList<>();
                    for(int i=0;i<jsonArray.size();i++){
                        String name = jsonArray.getString(i);
                        if(StringUtils.isEmpty(name)){
                            continue;
                        }
                        BaseEntity endEntity =  createNoExists(startLabel+property, name);
                        Long endId = endEntity.getId();
                        RelationshipVO relationshipVO = new RelationshipVO();
                        relationshipVO.setStartId(startId);
                        relationshipVO.setEndId(endId);
                        relationshipVO.setStartLabel(startLabel);
                        relationshipVO.setEndLabel(startLabel+property);
                        relationshipVO.setRelationshipType(startLabel+"相关"+property);
                        relationshipList.add(relationshipVO);
                    }
                    if(!CollectionUtils.isEmpty(relationshipList)) {
                        System.out.println(relationshipService.createRelationship(relationshipList));
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        }
    }

    private BaseEntity createNoExists(String labelName, String name) {
        BaseEntity nodeByName = entityService.findNodeByName(labelName, name);
        if(Objects.nonNull(nodeByName)){//节点不存在
            return nodeByName;
        }
        CreateEntityVO createEntity = new CreateEntityVO();
        createEntity.setName(name);
        createEntity.setLabel(labelName);
        return entityService.create(createEntity);
    }

    public static void main(String[] args) throws Exception {
        String propertys ="英文名称";
        workbook = new HSSFWorkbook();//这里也可以设置sheet的Name
        String accessToken = BaidubceUtil.getAccessToken();
        String[] split = propertys.split(",");
        for(String property:split) {
            HSSFSheet sheet = workbook.createSheet(property);
            saveExecl(property,accessToken, sheet);
        }
    }

    public static String readTxtFile(File txtFile) {
        StringBuilder content = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new FileReader(txtFile))) {
            String line;
            // 逐行读取文件内容
            while ((line = reader.readLine()) != null) {
                // 输出读取到的每一行内容
                content.append(line+"\n");
            }
        } catch (IOException e) {
            System.out.println("读取文件时发生错误: " + e.getMessage());
        }
        return content.toString();
    }

    private static List<Knowlege> saveExecl(String property, String accessToken, HSSFSheet sheet) throws Exception {
        int curCount = 0;
        List<Knowlege> result = new ArrayList<>();
        File directory = new File(directoryPath);
        // 获取目录下的所有文件和子目录
        File[] files = directory.listFiles();

        // 检查files是否为null，避免NullPointerException
        if (files != null) {
            // 循环遍历目录下的每个文件和子目录
            for (File file : files) {
                // 检查文件是否是.txt文件
                if (file.isFile() && file.getName().endsWith(".txt")) {
                    // 读取.txt文件的内容
                    String s = readTxtFile(file);
                    String name = file.getName().split("_")[1].replace(".txt","");
            /*        if(!s.contains(property)){
                        continue;
                    }*/
                    String value = getValues(s, property, accessToken);

                    HSSFRow writeRow = sheet.createRow(curCount);
                    name = (name == null ? "" : name);
                    value = (value == null ? "" : value);
                    writeRow.createCell(0).setCellValue(name);
                    writeRow.createCell(1).setCellValue(value);
                    writeRow.createCell(2).setCellValue(s == null ? "" : s);
                    Knowlege knowlege = new Knowlege();
                    knowlege.setEntity(name);
                    knowlege.setProperty(property);
                    knowlege.setValue(value);

                    result.add(knowlege);
                    curCount++;
                    if (maxCount > 0) {
                        if (curCount >= maxCount) {
                            break;
                        }
                    }
                }
            }
        } else {
            System.out.println("指定的目录不存在或无法访问: " + directoryPath);
        }
        save(property);
        return result;
    }

    private static String getValues(String zhaiyao, String property, String accessToken) {
        if (StringUtils.isEmpty(zhaiyao)) {
            return "";
        }
        String format = "你是医学检查领域的专家。你将在指定的文本中抽取其中“" + property + "”。\n" +
                "\n" +
                "#要求\n" +
                "1、抽取的结果将以JSON数组的形式呈现。每个抽取的“" + property + "”高度简洁、高度概括，不要要描述性的文字，文字尽量保持在20个字符以内！\n" +
                "\n" +
                "#示例1\n" +
                "以抽取“英文名称”为例\n" +
                "文本：\n" +
                "名称:饱餐试验\n" +
                "英文名称:full meal test\n" +
                "概述:SE是指异常癫痫样电活动持续发放，导致意识障碍、精神行为或认知功能异常，和（或）各种形式的惊厥发作持续时间超过5分钟或者发作间期未恢复到基线水平。根据有无运动症状，SE分为惊厥性癫痫持续状态（convulsive status epilepticus，CSE）和非惊厥性癫痫持续状态（non-convulsive status epilepticus，NCSE），前者包括全身强直-阵挛持续状态（generalized tonicclonic status epilepticus，GTSCE）和持续性部分性癫痫（epileptic partial continua，EPC）；后者包括4种主要临床类型：失神发作持续状态（absence status epilepticus，ASE）、单纯部分性发作持续状态（simple partial status epilepticus，SPSE）、复杂部分性发作持续状态（complex partial status epilepticus，CPSE）和昏迷中的癫痫持续状态（status epilepticus in coma），其中，昏迷中的癫痫持续状态还包括微小发作持续状态（subtle status epilepticus，SSE）。\n" +
                "\n" +
                "输出：[\"full meal test\"]\n\n"+
                "#示例2\n" +
                "以抽取“参考区间”为例\n" +
                "文本：\n" +
                "\n" +
                "名称:胃镜检查\n" +
                "英文名称:gastroscopy;stomachoscopy\n" +
                "概述:\n" +
                " \n" +
                "内镜（endoscopy）是从人体天然的开口部位（口腔、肛门、鼻腔等）或切口部位（腹部、胆道等）插入，用以窥视人体内部的一类仪器。自19世纪第一台内镜问世以来，从最初的硬式内镜至纤维内镜、电子内镜、胶囊内镜，至今已有100多年的历史。近年来，新的仪器设备和先进技术不断涌现，如色素内镜（chromoendoscopy）、放大内镜（magnifying \n" +
                "endoscopy）、荧光内镜（fluorescein electronic endoscopy）、红外线电子内镜（infrared electronic \n" +
                "endoscopy）、窄光谱光源结合放大内镜（narrow band light source combined with magnifying \n" +
                "endoscopy）、共聚焦激光显微内镜（laser scanning confocal \n" +
                "microscopy）等。检查的内容精细到对胃小凹、腺管开口、黏膜血管形态的观察，为消化道肿瘤早期诊断提供了有力的帮助，提高了早期癌变的诊断效率，也为内镜下治疗奠定了基础。内镜下开展的各种治疗技术，疗效确切，创伤小，开始部分取代传统的手术治疗。\n" +
                "\n" +
                "输出：[\"gastroscopy\",\"stomachoscopy\"]\n\n"+
                "2、没有可抽取的“" + property + "”，则返回空json数组。\n\n" +

                "本次抽取的文本如下：\n\n";

        String zhiling = format + zhaiyao;
        System.out.println(zhiling);
        String chatResponse = BaidubceUtil.getChatResponse(zhiling, accessToken);
        chatResponse = filte(chatResponse);
        System.out.println(chatResponse);
        return chatResponse;
    }

    private static synchronized void save(String fileName) {
        try {
            fileName="C:\\Users\\17664\\Desktop\\"+fileName+System.currentTimeMillis()+".xlsx";
            //文档输出
            FileOutputStream out = new FileOutputStream(new File(fileName));
            workbook.write(out);
            out.close();
            System.out.println(fileName + "存储完毕");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static String filte(String chatResponse) {
        if (chatResponse.startsWith("```json")) {
            chatResponse = chatResponse.substring(7);
        }
        if (chatResponse.endsWith("```")) {
            chatResponse = chatResponse.substring(0, chatResponse.length() - 3);
        }
        return chatResponse;
    }


}

