package com.qizhen.healsphere;

import cn.hutool.core.collection.CollectionUtil;
import com.qizhen.healsphere.repository.mapper.KgEdgesMapper;
import com.qizhen.healsphere.repository.mapper.KgNodesMapper;
import com.qizhen.healsphere.repository.mapper.KgSchemasMapper;
import com.qizhen.healsphere.repository.mapper.entity.KgEdges;
import com.qizhen.healsphere.repository.mapper.entity.KgNodes;
import com.qizhen.healsphere.repository.mapper.entity.KgSchemas;
import com.qizhen.healsphere.repository.neo4j.BaseRelationshipRepository;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.repository.neo4j.entity.Edge;
import com.qizhen.healsphere.util.DataMigrationUtil;
import com.qizhen.healsphere.util.DrugRelationCreator;
import com.qizhen.healsphere.util.IcdRelationCreator;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;

@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere"})
@SpringBootTest
public class MigrateDataTest {
    String version = "1.1";
    @Autowired
    BaseRelationshipRepository baseRelationshipRepository;

    @Autowired
    KgNodesMapper kgNodesMapper;

    @Autowired
    KgEdgesMapper kgEdgesMapper;

    @Autowired
    KgSchemasMapper kgSchemasMapper;

    @Autowired
    DataMigrationUtil dataMigrationUtil;

    @Autowired
    private IcdRelationCreator icdRelationCreator;

    @Autowired
    private DrugRelationCreator drugRelationCreator;

    @Test
    public void mergeData() {

        HashMap<String, Object> columnMap = new HashMap<>();
        columnMap.put("category", "药品");
        List<KgNodes> kgNodes = kgNodesMapper.selectByMap(columnMap);
        for(KgNodes temp:kgNodes) {
            columnMap = new HashMap<>();
            columnMap.put("name", temp.getName());
            columnMap.put("category", "Drug");
            List<KgNodes> drugs = kgNodesMapper.selectByMap(columnMap);
            if(!CollectionUtils.isEmpty(drugs)){
                if(drugs.size()>1){
                    System.out.println(1111);
                }
                kgEdgesMapper.update1(temp.getId(), drugs.get(0).getId());
                kgEdgesMapper.update2(temp.getId(), drugs.get(0).getId());
            }else{
                temp.setCategory("Drug");
                kgNodesMapper.updateById(temp);
            }
        }
    }

    @Test
    public void migrateData() {

        String endLabels = "ICD10编码";
        String[] split = endLabels.split(",");
        String startLabel = "疾病";
        addIfNotExist(startLabel);
        for(String endLabel:split) {
            String relationship = "疾病相关ICD10临床2.0编码";
            String name = "";


            List<Edge> edges = baseRelationshipRepository.triples(startLabel, relationship, endLabel);
            if (CollectionUtil.isEmpty(edges)) {
                continue;
            }
            if ("就诊科室".equals(endLabel)) {
                name = "所属科室";
                relationship = "belongs_to";
            } else if ("症状".equals(endLabel)) {
                relationship = "has_symptom";
                name = "症状";
            } else {
                name = relationship;
            }
            addIfNotExist(startLabel+endLabel);

            for (Edge temp : edges) {
                Integer startNodeId = addNode(temp.getStartNode());
                Integer endNodeId = addNode(temp.getEndNode());
                if (startNodeId <= 0 || endNodeId <= 0) {
                    continue;
                }
                HashMap<String, Object> columnMap = new HashMap<>();
                columnMap.put("src_id", startNodeId);
                columnMap.put("dest_id", endNodeId);
                columnMap.put("name", name);
                List<KgEdges> kgEdgesList = kgEdgesMapper.selectByMap(columnMap);
                if (CollectionUtil.isEmpty(kgEdgesList)) {
                    KgEdges kgEdges = new KgEdges();
                    kgEdges.setSrc_id(startNodeId);
                    kgEdges.setDest_id(endNodeId);
                    kgEdges.setName(name);
                    kgEdges.setCategory(relationship);
                    kgEdges.setVersion(version);
                    kgEdgesMapper.insert(kgEdges);
                }
            }
        }
    }

    @Test
    public void migrateDataByExist() {

        String relationships = "禁忌症";
        String[] split = relationships.split(",");
        String startLabel = "辅助检查";
        String endLabel = "疾病";
        addIfNotExist(startLabel);
        for(String relationship:split) {
            List<Edge> edges = baseRelationshipRepository.triples(startLabel, startLabel+"相关"+relationship, endLabel);
            if (CollectionUtil.isEmpty(edges)) {
                continue;
            }
            addIfNotExist(startLabel+relationship);

            for (Edge temp : edges) {
                Integer startNodeId = addNode(temp.getStartNode());
                Integer endNodeId = addNode(temp.getEndNode());
                if (startNodeId <= 0 || endNodeId <= 0) {
                    continue;
                }
                HashMap<String, Object> columnMap = new HashMap<>();
                columnMap.put("src_id", startNodeId);
                columnMap.put("dest_id", endNodeId);
                columnMap.put("name", relationship);
                List<KgEdges> kgEdgesList = kgEdgesMapper.selectByMap(columnMap);
                if (CollectionUtil.isEmpty(kgEdgesList)) {
                    KgEdges kgEdges = new KgEdges();
                    kgEdges.setSrc_id(startNodeId);
                    kgEdges.setDest_id(endNodeId);
                    kgEdges.setName(relationship);
                    kgEdges.setCategory(relationship);
                    kgEdges.setVersion(version);
                    kgEdgesMapper.insert(kgEdges);
                }
            }
        }
    }



    @Test
    public void migrateDataBySelf() {
        String relationships = "英文名称";
        String[] split = relationships.split(",");
        String startLabel = "辅助检查";
        String endLabel = startLabel;
        addIfNotExist(startLabel);
        for(String relationship:split) {
            List<Edge> edges = baseRelationshipRepository.triples(startLabel, startLabel+"相关"+relationship, endLabel);
            if (CollectionUtil.isEmpty(edges)) {
                continue;
            }
            addIfNotExist(startLabel+relationship);

            for (Edge temp : edges) {
                Integer startNodeId = addNode(temp.getStartNode());
                Integer endNodeId = addNode(temp.getEndNode());
                if (startNodeId <= 0 || endNodeId <= 0) {
                    continue;
                }
                HashMap<String, Object> columnMap = new HashMap<>();
                columnMap.put("src_id", startNodeId);
                columnMap.put("dest_id", endNodeId);
                columnMap.put("name", relationship);
                List<KgEdges> kgEdgesList = kgEdgesMapper.selectByMap(columnMap);
                if (CollectionUtil.isEmpty(kgEdgesList)) {
                    KgEdges kgEdges = new KgEdges();
                    kgEdges.setSrc_id(startNodeId);
                    kgEdges.setDest_id(endNodeId);
                    kgEdges.setName(relationship);
                    kgEdges.setCategory(relationship);
                    kgEdges.setVersion(version);
                    kgEdgesMapper.insert(kgEdges);
                }
            }
        }
    }

    private void addIfNotExist(String label){
        if("就诊科室".equals(label)) {
            label = "科室";
        }
        HashMap<String, Object> columnMap = new HashMap<>();
        columnMap.put("name", label);
        List<KgSchemas> kgSchemasList = kgSchemasMapper.selectByMap(columnMap);
        if(!CollectionUtil.isEmpty(kgSchemasList)) {
            return;
        }
        KgSchemas kgSchemas = new KgSchemas();
        kgSchemas.setName(label);
        kgSchemas.setCategory(label);
        kgSchemas.setContent("name");
        kgSchemas.setVersion(version);
        kgSchemasMapper.insert(kgSchemas);
    }

    private Integer addNode(BaseEntity node) {
        try {
            String category = node.getLabel().get(0);
            if("疾病".equals(category)){
                category = "Disease";
            }else if("就诊科室".equals(category)){
                category = "Department";
            }else if("症状".equals(category)){
                category = "Symptom";
            }
            HashMap<String, Object> columnMap = new HashMap<>();
            columnMap.put("name", node.getName());
            columnMap.put("category", category);
            List<KgNodes> kgNodesList = kgNodesMapper.selectByMap(columnMap);
            if (!CollectionUtil.isEmpty(kgNodesList)) {
                return kgNodesList.get(0).getId();
            }

            KgNodes kgNodes = new KgNodes();
            kgNodes.setName(node.getName());
            kgNodes.setCategory(category);
            kgNodes.setVersion(version);
            kgNodesMapper.insert(kgNodes);
            return kgNodes.getId();
        }catch (Exception e){
            e.printStackTrace();
            return -1;
        }
    }

    @Test
    public void migrateDataFromPostgresToNeo4j() {
        try {
            dataMigrationUtil.migrateByRelations();
            System.out.println("数据迁移成功");
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("数据迁移失败:" + e.getMessage());
        }
    }

    @Test
    public void createIcdRelations() {
        icdRelationCreator.createIcdRelations();
    }

    @Test
    public void createDrugSubclassRelations() {
        drugRelationCreator.createDrugSubclassRelations();
    }
}
